
#include <Poco/Logger.h>
#include <Poco/StringTokenizer.h>
#include <Poco/String.h>
#include "ChatLine.hpp"


using Poco::Logger;
using Poco::StringTokenizer;


ChatLine::ChatLine() : _params()
{
}


string ChatLine::command()
{
	if (_params.count("command") == 0)
		return "";
	return _params["command"];
}


void ChatLine::command(string cmd)
{
	_params["command"] = cmd;
}


string ChatLine::username()
{
	if (_params.count("username") == 0)
		return "";
	return _params["username"];	
}


void ChatLine::username(string user)
{
	_params["username"] = user;
}


string ChatLine::password()
{
	if (_params.count("password") == 0)
		return "";
	return _params["password"];	
}


void ChatLine::password(string pass)
{
	_params["password"] = pass;
}


void ChatLine::to(string t)
{
	_params["to"] = t;
}


string ChatLine::to()
{
	if (_params.count("to") == 0)
		return "";
	return _params["to"];
}


void ChatLine::from(string f)
{
	_params["from"] = f;
}


string ChatLine::from()
{
	if (_params.count("from") == 0)
		return "";
	return _params["from"];
}


void ChatLine::chatLine(string line)
{
	_params["chat"] = line;
}


string ChatLine::chatLine()
{
	if (_params.count("chat") == 0)
		return "";
	return _params["chat"];		
}


bool ChatLine::empty()
{
	return _params.empty();
}


string ChatLine::toString()
{
	string result;
	for (map<string, string>::const_iterator it = _params.begin(); it != _params.end(); it++)
		result += it->first + "=" + it->second + ",";
	return result;
}


bool ChatLine::parse(string request)
{
	StringTokenizer tokens(request, ChatLine::PARAM_SEP);
	for (StringTokenizer::Iterator it = tokens.begin(); it != tokens.end(); it++)
	{
		StringTokenizer nameValueToken(*it, ChatLine::NAME_VALUE_SEP);
		if (nameValueToken.count() % 2 != 0)
		{
			return false;
		}
		string name = Poco::trim<string>(nameValueToken[0]);
		string value = Poco::trim<string>(nameValueToken[1]);
		_params[name] = value;
	}
	return true;
}


string ChatLine::format()
{
	string str;
	for (map<string, string>::const_iterator it = _params.begin(); it != _params.end(); it++)
	{
		if (str == "")
			str = it->first + ChatLine::NAME_VALUE_SEP + it->second;
		else
			str += ChatLine::PARAM_SEP + it->first + ChatLine::NAME_VALUE_SEP + it->second;
	}
	return str;
}


const string ChatLine::PARAM_SEP = "&";
	
const string ChatLine::NAME_VALUE_SEP = "=";
