
#include <cassert>
#include <Poco/Thread.h>
#include <Poco/Logger.h>
#include <Poco/Format.h>
#include "Handler.hpp"


using Poco::Thread;
using Poco::Logger;


map<string, string> Handler::_members;


Handler::Handler()
{
}


void Handler::run(Message* message)
{
	Logger& log = Logger::get("server");
	
	// DONE status is signaled by closing socket
	if (message->session()->status() == Session::DONE)
	{
		removeUser(message->session()->id());
		message->session()->status(Session::DESTROY);
	}
	// check commands and their proper usage
	else if (message->chatLine()->command() == "login")
	{
		if (message->session()->status() == Session::NEW)
		{
			string username = message->chatLine()->username();
			string password = message->chatLine()->password();
			if (Handler::_members.count(username) > 0 && Handler::_members[username] == password)
			{
				log.debug("Handler::run(): session id=" +	Poco::format("%ld", message->session()->id()) + ", login ok");
				message->session()->status(Session::ACTIVE);
				_mutexUsers.lock();
				_users[message->session()->id()] = username;
				_mutexUsers.unlock();
				message->chatLine()->command("login ok");
			}
			else
			{
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", login failed");
				message->session()->status(Session::DONE);
				removeUser(message->session()->id());
				message->chatLine()->command("login failed");
			}
		}
		else
		{
			log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", cannot login");
			message->chatLine()->command("cannot login");
		}
	}
	else if (message->chatLine()->command() == "logout")
	{
		log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", logout");
		message->session()->status(Session::DONE);
		removeUser(message->session()->id());
		message->chatLine()->command("logout ok");
	}
	else if (message->chatLine()->command() == "send")
	{
		if (message->session()->status() == Session::ACTIVE)
		{
			if (message->chatLine()->to() == "")
			{
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", send failed, no 'to' field");
				message->chatLine()->command("send failed, no 'to' field");
			}
			else if (message->chatLine()->chatLine() == "")
			{
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", send failed, no 'chat' field");
				message->chatLine()->command("send failed, no 'chat' field");
			}
			else
			{
				_mutexUsers.lock();
				string userFrom = _users[message->session()->id()];
				_mutexUsers.unlock();
				_mutexInbox.lock();
				_inbox[message->chatLine()->to()].push_back(From(userFrom, message->chatLine()->chatLine()));
				_mutexInbox.unlock();
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", send ok, from=" +
					Poco::format("%s", message->chatLine()->from()) + ", chat=" + Poco::format("%s", message->chatLine()->chatLine()));
				message->chatLine()->command("send ok");
			}
		}
		else
		{
			log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", send failed, not authorized");
			message->session()->status(Session::DONE);
			removeUser(message->session()->id());
			message->chatLine()->command("send failed, not authorized");
		}
	}
	else if (message->chatLine()->command() == "receive")
	{
		if (message->session()->status() == Session::ACTIVE)
		{
			_mutexUsers.lock();
			string username = _users[message->session()->id()];
			_mutexUsers.unlock();
			_mutexInbox.lock();
			From fm;
			if (_inbox.count(username) > 0 && _inbox[username].size() > 0)
			{
				fm = _inbox[username].back();
				_inbox[username].pop_back();
			}
			_mutexInbox.unlock();
			if (fm.empty())
			{
				message->chatLine()->command("receive ok, no chatLine");
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", receive ok, no chatLine");
			}
			else
			{
				message->chatLine()->command("receive ok");
				message->chatLine()->chatLine(fm.chatLine);
				message->chatLine()->from(fm.from);
				log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", receive ok, from=" +
					Poco::format("%s", fm.from) + ", chat=" + Poco::format("%s", fm.chatLine));
			}
		}
		else
		{
			log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", receive failed, not authorized");
			message->session()->status(Session::DONE);
			removeUser(message->session()->id());
			message->chatLine()->command("receive failed, not authorized");
		}
	}
	else
	{
		log.debug("Handler::run(): session id=" + Poco::format("%ld", message->session()->id()) + ", unknown command");
		message->chatLine()->command("unknown command");
	}
}


/**
Removes user's session.
**/
void Handler::removeUser(long sessionId)
{
	_mutexUsers.lock();
	_users.erase(sessionId);
	_mutexUsers.unlock();
}
