
#include "ServerConnection.hpp"
#include <Poco/Format.h>
#include <Poco/Logger.h>


using Poco::Logger;


Handler ServerConnection::_handler;


ServerConnection::ServerConnection(const StreamSocket& client) : TCPServerConnection(client)
{
}


void ServerConnection::run()
{
	Logger& log = Logger::get("server");

	Message* message = Message::create();
	message->session()->status(Session::NEW);
	log.information("ServerConnection::run(): session " + Poco::format("%ld", message->session()->id()) + " created");

	while (true)
	{
		const short BUF_SIZE = 128;
		char buf[BUF_SIZE];
		int result = this->socket().receiveBytes(reinterpret_cast<void*>(buf), BUF_SIZE - 1);
		if (result == 0)
		{
			message->session()->status(Session::DONE);
		}
		else
		{
			buf[result] = '\0';
			string request = buf;
			ChatLine* line = new ChatLine;
			line->parse(request);
			message->chatLine(line);
		}

		ServerConnection::_handler.run(message);
		if (message->session()->status() == Session::DESTROY)
			break;
		
		string response = message->chatLine()->format();
		result = this->socket().sendBytes(reinterpret_cast<const void*>(response.c_str()), response.size());
		if (result == 0)
		{
			// have to set DONE status so processor could remove the logged user
			break;
		}
	}
	
	log.information("ServerConnection::run(): session " + Poco::format("%ld", message->session()->id()) + " destroyed");
	Message::destroy(message);
}
